"""
(c) 2020 Amazon Web Services, Inc. or its affiliates. All Rights Reserved. 
This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at  or other written agreement between Customer and Amazon Web Services, Inc.

EMR Security Certification
Description: The function checks whether S3-logging is enabled for an EMR cluster.

Services: Cloudwatch Events (trigger), EMR, SNS
Runtime: Python 3.6 
"""

import logging
import os

import boto3
from botocore.exceptions import ClientError

EMR_CLIENT = boto3.client('emr')
OUTBOUND_TOPIC_ARN = os.environ["outbound_topic_arn"]


def lambda_handler(event, context):
    """
    Lambda event handler that processes event received
    """
    setup_logging()
    log.info('Got an event!')
    log.info(event)

    if "errorMessage" in event["detail"]["responseElements"]:
        log.error("Event carried an error message. Aborting")
        return

    if event["detail"]["eventName"] in ["RunJobFlow"]:
        violations = []
        try:
            if not s3_logging(event):
                violations.append("Violation - S3 Logging Disabled")
        except Exception as err:
            log.error(err)
            raise

        try:
            if violations:
                subject = "Violation - EMR Cluster is Non-Compliant"
                message = create_non_compliance_message(
                    violations, event)
                send_violation(OUTBOUND_TOPIC_ARN, message, subject, event, context)
            else:
                log.info("No violations found.")
        except Exception as err:
            log.error(err)
            raise


def s3_logging(event):
    """
    The function checks to validate whether S3 logging for an EMR cluster
    is enabled or not.
    """
    try:
        if "logUri" in event["detail"]["requestParameters"]:
            return True
        return False
    except Exception as err:
        log.error(err)
        raise


def create_non_compliance_message(violations, event):
    """
    The function creates a non-compliance message to be notified.
    """
    try:
        message = "Violation - The following EMR Cluster was detected to be in violation: \n\n"
        message += "EMR Cluster ID : " + event["detail"]["responseElements"]["jobFlowId"] + "\n"
        for violation in violations:
            message += violation + "\n"
        return message
    except Exception as err:
        log.error(err)
        raise


def send_violation(outbound_topic_arn, message, subject, event, context):
    """
    The function sends violation message
    """
    findsnsregion = outbound_topic_arn.split(":")
    snsregion = findsnsregion[3]
    message += '\nAccount: ' + event["account"] + "\n"
    message += "Region: " + event["detail"]["awsRegion"] + "\n"
    message += "\n\n"
    message += "This notification was generated by the Lambda function " + \
               context.invoked_function_arn
    sendclient = boto3.client('sns', region_name=snsregion)
    try:
        sendclient.publish(
            TopicArn=outbound_topic_arn,
            Message=message,
            Subject=subject
        )
    except ClientError as err:
        log.error(err)


def setup_logging():
    """
    Logging Function.
    Creates a global log object and sets its level.
    """
    global log
    log = logging.getLogger()
    log_levels = {'INFO': 20, 'WARNING': 30, 'ERROR': 40}

    if 'logging_level' in os.environ:
        log_level = os.environ['logging_level'].upper()
        if log_level in log_levels:
            log.setLevel(log_levels[log_level])
        else:
            log.setLevel(log_levels['ERROR'])
            log.error("The logging_level environment variable is not set to INFO, WARNING, or \
                        ERROR.  The log level is set to ERROR")
    else:
        log.setLevel(log_levels['ERROR'])
        log.warning('The logging_level environment variable is not set. The log level is set to \
                    ERROR')
    log.info('Logging setup complete - set to log level ' + str(log.getEffectiveLevel()))
